package com.ashokit.ticketbooking.controller;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.ashokit.ticketbooking.model.Ticket;
import com.ashokit.ticketbooking.service.TicketService;

@RestController //@Rest + @controller ==> TicketController exposed to outside world
@RequestMapping(value="/tickets") //http://localhost:8080/tickets==> It will bootstrap TicketController
								  //obj=ioc.getTicketController();


/*
 * 
 * 
 * HOW REST PROTOCOL IS DESIGNED ==> HETEROGENIOUS SYSTEMS COMMUNICATION ==> JSON/XML
 * 
 *   CRETE THE DATA ==> @POST 
 *   UPDATE THE DATA ==> @PUT
 *   RETRIEVE THE DATA ==> @GET
 *   DELETE THE DATA ==> @DELETE
 *   
 *   
 *   @Post+@RequestMapping ==> @PostMapping
 *   @Put+@RequestMapping ==> @PutMapping
 *   @Get+@RequestMapping ==> @GetMapping
 *   @Delete+@RequestMapping ==> @DeleteMapping
 * 
 * JAVA TO JSON CONVERSION REQUIRED ==> spring boot is giving data to outside world ==> MARSHALLING
 * 
 * JSON TO JAVA CONVERSION REQUIRED ===> outside world giving data to spring boot ==> UNMARSHALLING
 * 
 * 
 * DEVELOPER NO NEED TO WRITE CODE ==> IT CREATED JACKSON LIBRARY ==> IT WILL TAKE CARE OF MARSHALLING AND UNMARSHALLING
 * 
 * @RequestBody ==> It will go to body section of the request and takes the JSON object and call marshalling api
 */
public class TicketController {
	
	
	@Autowired
	private TicketService ticketService;
	
	//create ticket
	
	@PostMapping(value="/create") 
	//http://localhost:8080/tickets/create
	public Ticket createTicket(@RequestBody Ticket inputObj) {
		return ticketService.createTicket(inputObj);
	}
	
	
	
	
	
	
	//retrieve all tickets
	//http://localhost:8080/ticktets/all==> obj.retrieveAllTickets();
	@GetMapping(value="/all")
	public Iterable<Ticket> retrieveAllTickets(){
		return ticketService.getAllTickets();
	}
	
	
	//retrieve single ticket
	//http://localhost:8080/tickets/1 ==> 1st Ticket
	//http://localhost:8080/tickets/2 ==> 2nd Ticket
	//http://localhost:8080/tickets/3 ==> rd Ticket
	
	/* @GetMapping(value="/{ticketId}")
	public Ticket getTicket(@PathVariable("ticketId")  Integer ticketId) {
		return ticketService.getTicket(ticketId);
	}*/
	
	
	@GetMapping(value="/{passengerName}")
	public Ticket getTicket(@PathVariable("passengerName")  String passengerName) {
		return ticketService.getByPassengerName(passengerName);
	}
	
	
	
	//update ticket
	
	//http://localhost:8080/tickets/1/a@b.com ==> 
	@PutMapping(value="/{ticketId}/{newEmail}")
	public Ticket updateTicket(@PathVariable("ticketId") Integer ticketId,
			@PathVariable("newEmail") String newEmail) {
		return ticketService.updateTicket(ticketId, newEmail);
	}
	
	//delete ticket
	//http://localhost:8080/tickets/1
	@DeleteMapping(value="/{ticketId}")
	public void deleteTicket(@PathVariable("ticketId")  Integer ticketId) {
		ticketService.deleteTicket(ticketId);
	}

}
