package com.ashokit.ticketbooking.service;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.ashokit.ticketbooking.dao.TicketDao;
import com.ashokit.ticketbooking.model.Ticket;

//Service Layer for business logic
@Service //Automatically it will initiate AOP
public class TicketService {

	
	
	//TicketDao obj=new TicketDao();
	
	@Autowired
	private TicketDao ticketDao;
	
	
	
	
	public Ticket getByPassengerName(String passengerName) {
		return ticketDao.findByPassengerName(passengerName);
	}
	
	
	//Business Logic
	
	
	//Create Ticket
	
	public Ticket createTicket(Ticket ticketObj) {
		return ticketDao.save(ticketObj);
		
		/*
		 * 
		 * save ==> Insert the Data/ Update The Data
		 * 
		 * if( ticketObj.getTicketId() == null) or 
		 * 					(ticketObj.getTicketId() != null but ticket id not exists in database)
		 * 
		 *      insert Query
		 *      
		 *      else
		 *      
		 *      update Query
		 */
	}
	
	
	//Retrieve All Tickets
	
	public Iterable<Ticket> getAllTickets(){
		return ticketDao.findAll();
		
		//findAll ==> select * from tbl_ticket;
		//          ==> Result set converted into Iterable<Ticket>
	}
	
	
	//Retrieve Individual Ticket
	
	public Ticket getTicket(Integer ticketId) {
		return ticketDao.findById(ticketId).orElse(new Ticket());
		
		//findById ==> select * from tbl_ticket where ticket_id=?
		// ==> Result set converted into Ticket
	}
	
	
	//Update Ticket
	
	public Ticket updateTicket(Integer ticketId,String newEmail) {
		
		Ticket dbObj=getTicket(ticketId);
		dbObj.setEmail(newEmail);
		
		return ticketDao.save(dbObj);
		//Here save is going to trigger update query
		
		//update tbl_ticket set .....
		
	}
	
	
	
	
	//Delete Ticket
	
	public void deleteTicket(Integer ticketId) {
		ticketDao.deleteById(ticketId);
		//deleteById ==> delete from tbl_ticket where ticket_id=ticketId
	}
}
